/**
 * Logback: the reliable, generic, fast and flexible logging framework.
 * 
 * Copyright (C) 1999-2007, QOS.ch
 * 
 * This library is free software, you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation.
 */
package ch.qos.logback.eclipse.actions;

/**
 * @author S&eacute;bastien Pennec
 */
import org.eclipse.jdt.internal.debug.ui.console.JavaStackTraceConsole;
import org.eclipse.jface.action.Action;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;

import ch.qos.logback.classic.spi.LoggingEvent;
import ch.qos.logback.classic.spi.ThrowableDataPoint;
import ch.qos.logback.eclipse.views.LogbackView;

public class OpenStackTraceAction extends Action {

  private static final String NEW_LINE = System.getProperty("line.separator");  //$NON-NLS-1$

  LogbackView view;
  JavaStackTraceConsole console;

  public OpenStackTraceAction(LogbackView view, String text) {
    super(text);
    this.view = view;
    initConsole();
  }

  @Override
  public void run() {
    LoggingEvent event = view.getSelectedEvent();
    if (event == null || event.getThrowableProxy() == null) {
      return;
    }

    StringBuffer buf = new StringBuffer();
    ThrowableDataPoint[] lines = event.getThrowableProxy().getThrowableDataPointArray();
    for (int i = 0; i < lines.length; i++) {
      buf.append(lines[i]).append(NEW_LINE);
    }

    //check that the user did not remove our console from the ConsoleView
    boolean stillRegistered = checkRegistration();
    if (!stillRegistered) {
      registerConsole();
    }
    
    console.clearConsole();
    console.getDocument().set(buf.toString());
    ConsolePlugin.getDefault().getConsoleManager().showConsoleView(console);
  }
  
  private boolean checkRegistration() {
    IConsole[]consoleArray = ConsolePlugin.getDefault().getConsoleManager().getConsoles();
    for (int i = 0; i < consoleArray.length; i++) {
      IConsole c = consoleArray[i];
      if (c == console) {
        return true;
      }
    }
    return false;
  }
  
  private void registerConsole() {
    ConsolePlugin.getDefault().getConsoleManager().addConsoles(new IConsole[] { console });
  }
  
  private void initConsole() {
    console = new JavaStackTraceConsole();
    ConsolePlugin.getDefault().getConsoleManager().addConsoles(new IConsole[] { console });
  }

}
