/**
 * Logback: the reliable, generic, fast and flexible logging framework.
 * 
 * Copyright (C) 1999-2007, QOS.ch
 * 
 * This library is free software, you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation.
 */
package ch.qos.logback.eclipse.reciever;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

import org.eclipse.osgi.util.NLS;
import org.eclipse.ui.IMemento;

import ch.qos.logback.eclipse.Messages;
import ch.qos.logback.eclipse.PluginLogger;

/**
 * A simple {@link SocketNode} based server that recieves the logging events and
 * redirects them to the LoggingEventManager
 * 
 * @author Ceki G&uuml;lc&uuml;
 * @author S&eacute;bastien Pennec
 */
public class SimpleSocketServer implements Runnable {

  private static final String TAG_SERVER = "lbserver"; //$NON-NLS-1$
  private static final String TAG_PORT = "port"; //$NON-NLS-1$
  
  private int port = 4321;
  private ServerSocket serverSocket;
  private boolean stop = false;

  public SimpleSocketServer() {
  }

  public void run() {
    startAndLoop();
  }

  public void setPort(int port) {
    this.port = port;
  }

  public int getPort() {
    return port;
  }

  public void restart() {
    try {
      serverSocket.close();
      stop = false;
      startAndLoop();
    } catch (IOException e) {
      PluginLogger.logError(e);
    }
  }

  private void startAndLoop() {
    stop = false;
    try {
      serverSocket = new ServerSocket(port);
      while (!stop) {
        Socket socket = serverSocket.accept();
        new Thread(new SocketNode(socket)).start();
      }
      serverSocket.close();
    } catch (java.net.BindException e) {
      PluginLogger.logError(NLS.bind(Messages.BindException, port), e);
    } catch (Exception e) {
      PluginLogger.logError(e);
    }
  }

  public void saveState(IMemento memento) {
    IMemento mem = memento.createChild(TAG_SERVER);
    mem.putInteger(TAG_PORT, port);
  }

  public void init(IMemento memento) {
    IMemento mem = memento.getChild(TAG_SERVER);
    if (mem == null) {
      return;
    }

    int newPort = mem.getInteger(TAG_PORT);

    if (newPort != 0) {
      this.port = newPort;
    }
  }
  
  public static boolean isPortFree(Integer port) {
    try {
      Socket s = new Socket("localhost", port); //$NON-NLS-1$
      return !s.isConnected();
    } catch (Exception e) {
      return true;
    }
  }
  
  public void stop() {
    stop = true;
  }

}
