/**
 * Logback: the reliable, generic, fast and flexible logging framework.
 * 
 * Copyright (C) 1999-2007, QOS.ch
 * 
 * This library is free software, you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation.
 */
package ch.qos.logback.eclipse.views;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.graphics.Image;

import ch.qos.logback.classic.boolex.JaninoEventEvaluator;
import ch.qos.logback.core.filter.EvaluatorFilter;
import ch.qos.logback.eclipse.Messages;
import ch.qos.logback.eclipse.model.EventFilter;

/**
 * @author S&eacute;bastien Pennec
 */
public class FilterContentProvider extends LabelProvider implements ITableLabelProvider,
    IStructuredContentProvider {

  private static FilterContentProvider provider = new FilterContentProvider();

  private TableViewer viewer;

  public static FilterContentProvider getProvider() {
    return provider;
  }

  public Object[] getElements(Object inputElement) {
    return EventFilter.getAllFilters().toArray();
  }

  public void setViewer(TableViewer viewer) {
    this.viewer = viewer;
  }

  public void dispose() {
  }

  public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
  }

  public Image getColumnImage(Object element, int columnIndex) {
    return null;
  }

  public String getColumnText(Object element, int columnIndex) {
    EvaluatorFilter filter = (EvaluatorFilter) element;
    JaninoEventEvaluator eval = (JaninoEventEvaluator) filter.getEvaluator();
    switch (columnIndex) {
    case 0:
      return eval.getExpression();
    case 1:
      return filter.getOnMatch().toString();
    case 2:
      return filter.getOnMismatch().toString();
    }

    return null;
  }

  public void remove(EvaluatorFilter filter) {
    EventFilter.remove(filter);
    viewer.remove(filter);
    viewer.refresh();
  }

  public EvaluatorFilter createNewFilter() {
    EvaluatorFilter current = new EvaluatorFilter();
    current.setContext(EventFilter.getContext());
    current.setName(Messages.EvaluatorFilterName);
    JaninoEventEvaluator eval = new JaninoEventEvaluator();
    eval.setContext(EventFilter.getContext());
    eval.setName(Messages.EvaluatorFilterEvalName);
    current.setEvaluator(eval);
    return current;
  }

}
